{% extends "admin/base_site.html" %}
{% load i18n static %}
{% load admin_urls %}

{% block extrahead %}{{ block.super }}
<script type="text/javascript" src="{% url 'admin:jsi18n' %}"></script>
{% endblock %}
{% block extrastyle %}{{ block.super }}<link rel="stylesheet" type="text/css" href="{% static "admin/css/forms.css" %}">{% endblock %}
{% block bodyclass %}{{ block.super }} {{ opts.app_label }}-{{ opts.model_name }} change-form{% endblock %}
{% if not is_popup %}
{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'admin:app_list' app_label=opts.app_label %}">{{ opts.app_config.verbose_name }}</a>
&rsaquo; <a href="{% url opts|admin_urlname:'changelist' %}">{{ opts.verbose_name_plural|capfirst }}</a>
&rsaquo; <a href="{% url opts|admin_urlname:'change' original.pk|admin_urlquote %}">{{ original|truncatewords:"18" }}</a>
&rsaquo; {% trans 'Change password' %}
</div>
{% endblock %}
{% endif %}
{% block content %}<div id="content-main">
<form action="{{ form_url }}" method="post" id="{{ opts.model_name }}_form">{% csrf_token %}{% block form_top %}{% endblock %}
<input type="text" name="username" value="{{ original.get_username }}" style="display: none">
<div>
{% if is_popup %}<input type="hidden" name="_popup" value="1">{% endif %}
{% if form.errors %}
    <p class="errornote">
    {% if form.errors.items|length == 1 %}{% trans "Please correct the error below." %}{% else %}{% trans "Please correct the errors below." %}{% endif %}
    </p>
{% endif %}

<p>{% blocktrans with username=original %}Enter a new password for the user <strong>{{ username }}</strong>.{% endblocktrans %}</p>

<fieldset class="module aligned">

<div class="form-row">
  {{ form.password1.errors }}
  {{ form.password1.label_tag }} {{ form.password1 }}
  {% if form.password1.help_text %}
  <div class="help">{{ form.password1.help_text|safe }}</div>
  {% endif %}
</div>

<div class="form-row">
  {{ form.password2.errors }}
  {{ form.password2.label_tag }} {{ form.password2 }}
  {% if form.password2.help_text %}
  <div class="help">{{ form.password2.help_text|safe }}</div>
  {% endif %}
</div>

</fieldset>

<div class="submit-row">
<input type="submit" value="{% trans 'Change password' %}" class="default">
</div>

</div>
</form></div>
{% endblock %}
