{% extends "admin/base_site.html" %}
{% load i18n %}

{% block breadcrumbs %}
<div class="breadcrumbs">
<a href="{% url 'admin:index' %}">{% trans 'Home' %}</a>
&rsaquo; <a href="{% url 'django-admindocs-docroot' %}">{% trans 'Documentation' %}</a>
&rsaquo; <a href="{% url 'django-admindocs-views-index' %}">{% trans 'Views' %}</a>
&rsaquo; {{ name }}
</div>
{% endblock %}
{% block title %}{% blocktrans %}View: {{ name }}{% endblocktrans %}{% endblock %}

{% block content %}

<h1>{{ name }}</h1>

<h2 class="subhead">{{ summary|striptags }}</h2>

{{ body }}

{% if meta.Context %}
<h3>{% trans 'Context:' %}</h3>
<p>{{ meta.Context }}</p>
{% endif %}

{% if meta.Templates %}
<h3>{% trans 'Templates:' %}</h3>
<p>{{ meta.Templates }}</p>
{% endif %}

<p class="small"><a href="{% url 'django-admindocs-views-index' %}">&lsaquo; {% trans 'Back to View documentation' %}</a></p>
{% endblock %}
